<?php

namespace FiloBlu\ExtInventory\Model;

use FiloBlu\ExtInventory\Api\Data\InventoryFbOrderItemSourceInterface;
use FiloBlu\ExtInventory\Api\Data\InventoryFbOrderitemSourceInterfaceFactory;
use FiloBlu\ExtInventory\Model\ResourceModel\InventoryFbOrderItemSource\Collection;
use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Model\AbstractModel;
use Magento\Framework\Model\Context;
use Magento\Framework\Registry;

/**
 * Class InventoryFbOrderItemSource
 * @package FiloBlu\ExtInventory\Model
 */
class InventoryFbOrderItemSource extends AbstractModel implements InventoryFbOrderItemSourceInterface
{
    /**
     * @var DataObjectHelper
     */
    protected $dataObjectHelper;
    /**
     * @var string
     */
    protected $_eventPrefix = 'inventory_fb_orderitem_source';
    /**
     * @var InventoryFbOrderitemSourceInterfaceFactory
     */
    protected $inventory_fb_orderitem_sourceDataFactory;

    /**
     * @param Context $context
     * @param Registry $registry
     * @param InventoryFbOrderitemSourceInterfaceFactory $inventory_fb_orderitem_sourceDataFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param ResourceModel\InventoryFbOrderItemSource $resource
     * @param Collection $resourceCollection
     * @param array $data
     */
    public function __construct(
        Context $context,
        Registry $registry,
        InventoryFbOrderitemSourceInterfaceFactory $inventory_fb_orderitem_sourceDataFactory,
        DataObjectHelper $dataObjectHelper,
        ResourceModel\InventoryFbOrderItemSource $resource,
        Collection $resourceCollection,
        array $data = []
    ) {
        $this->inventory_fb_orderitem_sourceDataFactory = $inventory_fb_orderitem_sourceDataFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * Retrieve inventory_fb_orderitem_source model with inventory_fb_orderitem_source data
     * @return InventoryFbOrderItemSourceInterface
     */
    public function getDataModel()
    {
        $inventory_fb_orderitem_sourceData = $this->getData();

        $inventory_fb_orderitem_sourceDataObject = $this->inventory_fb_orderitem_sourceDataFactory->create();
        $this->dataObjectHelper->populateWithArray(
            $inventory_fb_orderitem_sourceDataObject,
            $inventory_fb_orderitem_sourceData,
            InventoryFbOrderItemSourceInterface::class
        );

        return $inventory_fb_orderitem_sourceDataObject;
    }

    /**
     * @inheritDoc
     */
    public function getInventoryFbOrderitemSourceId()
    {
        return $this->getData(InventoryFbOrderItemSourceInterface::INVENTORY_FB_ORDERITEM_SOURCE_ID);
    }

    /**
     * @inheritDoc
     */
    public function setInventoryFbOrderitemSourceId($inventoryFbOrderitemSourceId)
    {
        $this->setData(InventoryFbOrderItemSourceInterface::INVENTORY_FB_ORDERITEM_SOURCE_ID, $inventoryFbOrderitemSourceId);
    }

    /**
     * @inheritDoc
     */
    public function getOrderId()
    {
        return $this->getData(InventoryFbOrderItemSourceInterface::ORDER_ID);
    }

    /**
     * @inheritDoc
     */
    public function setOrderId($orderId)
    {
        $this->setData(InventoryFbOrderItemSourceInterface::ORDER_ID, $orderId);
    }

    /**
     * @inheritDoc
     */
    public function getOrderItemId()
    {
        return $this->getData(InventoryFbOrderItemSourceInterface::ORDER_ITEM_ID);
    }

    /**
     * @inheritDoc
     */
    public function setOrderItemId($orderItemId)
    {
        $this->setData(InventoryFbOrderItemSourceInterface::ORDER_ITEM_ID, $orderItemId);
    }

    /**
     * @inheritDoc
     */
    public function getInventorySourceId()
    {
        return $this->getData(InventoryFbOrderItemSourceInterface::INVENTORY_SOURCE_ID);
    }

    /**
     * @inheritDoc
     */
    public function setInventorySourceId($inventorySourceId)
    {
        $this->setData(InventoryFbOrderItemSourceInterface::INVENTORY_SOURCE_ID, $inventorySourceId);
    }

    /**
     * @inheritDoc
     */
    public function getFinalInventorySourceId()
    {
        return $this->getData(InventoryFbOrderItemSourceInterface::FINAL_INVENTORY_SOURCE_ID);
    }

    /**
     * @inheritDoc
     */
    public function setFinalInventorySourceId($finalInventorySourceId)
    {
        $this->setData(InventoryFbOrderItemSourceInterface::FINAL_INVENTORY_SOURCE_ID, $finalInventorySourceId);
    }

    /**
     * @inheritDoc
     */
    public function getSku()
    {
        return $this->getData(InventoryFbOrderItemSourceInterface::SKU);
    }

    /**
     * @inheritDoc
     */
    public function setSku($sku)
    {
        $this->setData(InventoryFbOrderItemSourceInterface::SKU, $sku);
    }

    /**
     * @inheritDoc
     */
    public function getQty()
    {
        return $this->getData(InventoryFbOrderItemSourceInterface::QTY);
    }

    /**
     * @inheritDoc
     */
    public function setQty($qty)
    {
        $this->setData(InventoryFbOrderItemSourceInterface::QTY, $qty);
    }
}
